@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2017-2021 Martijn Braam & Clayton Craft <clayton@craftyguy.net>
+Copyright (C) 2017-2022 Martijn Braam & Clayton Craft <clayton@craftyguy.net>
 
 This file is part of osk-sdl.
 
@@ -157,6 +157,22 @@ bool Config::Read(const std::string &path)
 		Config::inputBoxRadius = Config::options["inputbox-radius"];
 	}
 
+	it = Config::options.find("inputbox-dot-glyph");
+	if (it != Config::options.end()) {
+		std::string val = Config::options["inputbox-dot-glyph"];
+		transform(val.begin(), val.end(), val.begin(), ::tolower);
+		if (val.compare("none") == 0) {
+			Config::inputBoxDotGlyph = "";
+		} else {
+			Config::inputBoxDotGlyph = Config::options["inputbox-dot-glyph"];
+		}
+	}
+
+	it = Config::options.find("inputbox-font-size");
+	if (it != Config::options.end()) {
+		Config::inputBoxFontSize = std::stoi(Config::options["inputbox-font-size"]);
+	}
+
 	it = Config::options.find("animations");
 	if (it != Config::options.end()) {
 		Config::animations = (Config::options["animations"] == "true");
@@ -175,24 +191,26 @@ bool Config::Parse(std::istream &file)
 		std::istringstream iss(line);
 		std::string id, eq, val;
 
-		bool error = false;
-
-		if (!(iss >> id)) {
-			continue;
-		} else if (id[0] == '#') {
+		iss >> id;
+		if (iss.fail()) {
 			continue;
 		} else if (id.empty()) {
 			continue;
-		} else if (!(iss >> eq >> val >> std::ws) || eq != "=" || iss.get() != EOF) {
-			error = true;
+		} else if (id[0] == '#') {
+			continue;
 		}
 
-		if (error) {
+		iss >> eq >> val;
+		// check that:
+		// 1) fail/bad bits aren't set
+		// 2) eq field is '='
+		// 3) that there are no trailing fields, after ignoring any trailing whitespace
+		if (iss.fail() || eq.compare("=") || !(iss >> std::ws).eof()) {
 			SDL_LogError(SDL_LOG_CATEGORY_ERROR, "Syntax error on line %d", lineno);
 			return false;
-		} else {
-			Config::options[id] = val;
 		}
+
+		Config::options[id] = val;
 	}
 	return true;
 }
