@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2017  Ian Shehadeh, Martijn Braam,
+Copyright (C) 2017-2022  Ian Shehadeh, Martijn Braam,
 & Clayton Craft <clayton@craftyguy.net>
 
 This file is part of osk-sdl.
@@ -34,7 +34,7 @@ SDL_Point *bezier_corner(SDL_Point *pts, SDL_Point *offset, SDL_Point *p1, SDL_P
 
 void smooth_corners(SDL_Rect *rect, int radius, const std::function<void(int, int)> &draw_cb)
 {
-	SDL_Point *corner = (SDL_Point *)malloc(sizeof(SDL_Point) * BEZIER_RESOLUTION);
+	SDL_Point *corner = static_cast<SDL_Point*>(malloc(sizeof(SDL_Point) * BEZIER_RESOLUTION));
 	//Top Left
 	bezier_corner(corner, new SDL_Point { rect->x - 1, rect->y - 1 },
 		new SDL_Point { 0, radius },
@@ -88,7 +88,7 @@ void smooth_corners_surface(SDL_Surface *surface, Uint32 color, SDL_Rect *rect,
 		}
 		Uint8 *pixel = static_cast<Uint8 *>(surface->pixels);
 		pixel += (y * surface->pitch) + (x * sizeof(Uint32));
-		*((Uint32 *)pixel) = color;
+		*(reinterpret_cast<Uint32 *>(pixel)) = color;
 	});
 }
 
