@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2017-2021
+Copyright (C) 2017-2022
 Martijn Braam, Clayton Craft <clayton@craftyguy.net>, et al.
 
 This file is part of osk-sdl.
@@ -166,6 +166,8 @@ void Keyboard::draw(SDL_Renderer *renderer, int screenHeight)
 
 		// Fill rounded corners at intersection
 		if (highlightedKey.isPreviewEnabled && keyRadius > 0) {
+			uint8_t r, g, b, a;
+			SDL_GetRenderDrawColor(renderer, &r, &g, &b, &a);
 			SDL_SetRenderDrawColor(renderer, config->keyBackgroundHighlighted.r, config->keyBackgroundHighlighted.g,
 				config->keyBackgroundHighlighted.b, config->keyBackgroundHighlighted.a);
 			SDL_Rect cornerRect;
@@ -174,6 +176,7 @@ void Keyboard::draw(SDL_Renderer *renderer, int screenHeight)
 			cornerRect.w = highlightSrcRect.w;
 			cornerRect.h = 2 * keyRadius;
 			SDL_RenderFillRect(renderer, &cornerRect);
+			SDL_SetRenderDrawColor(renderer, r, g, b, a);
 		}
 
 		// Draw highlighted key & preview
@@ -454,24 +457,32 @@ void Keyboard::loadKeymap()
 	layer0.rows[2] = { "a", "s", "d", "f", "g", "h", "j", "k", "l" };
 	layer0.rows[3] = { "z", "x", "c", "v", "b", "n", "m" };
 	layer0.layerNum = 0;
+	layer0.texture = nullptr;
+	layer0.highlightedTexture = nullptr;
 
 	layer1.rows[0] = { "!", "@", "#", "$", "%", "^", "&", "*", "(", ")" };
 	layer1.rows[1] = { "Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P" };
 	layer1.rows[2] = { "A", "S", "D", "F", "G", "H", "J", "K", "L" };
 	layer1.rows[3] = { "Z", "X", "C", "V", "B", "N", "M" };
 	layer1.layerNum = 1;
+	layer1.texture = nullptr;
+	layer1.highlightedTexture = nullptr;
 
 	layer2.rows[0] = {};
 	layer2.rows[1] = { "1", "2", "3", "4", "5", "6", "7", "8", "9", "0" };
 	layer2.rows[2] = { "@", "#", "$", "%", "&", "-", "_", "+", "(", ")" };
 	layer2.rows[3] = { ",", "\"", "'", ":", ";", "!", "?" };
 	layer2.layerNum = 2;
+	layer2.texture = nullptr;
+	layer2.highlightedTexture = nullptr;
 
 	layer3.rows[0] = {};
 	layer3.rows[1] = { "~", "`", "|", "·", "√", "π", "τ", "÷", "×", "¶" };
 	layer3.rows[2] = { "©", "®", "£", "€", "¥", "^", "°", "*", "{", "}" };
 	layer3.rows[3] = { "\\", "/", "<", ">", "=", "[", "]" };
 	layer3.layerNum = 3;
+	layer3.texture = nullptr;
+	layer3.highlightedTexture = nullptr;
 
 	keyboard.push_back(layer0);
 	keyboard.push_back(layer1);
@@ -484,6 +495,7 @@ touchArea Keyboard::getKeyForCoordinates(int x, int y)
 	for (const auto &layer : keyboard) {
 		if (layer.layerNum == activeLayer) {
 			for (const auto &it : layer.keyVector) {
+				// cppcheck-suppress useStlAlgorithm
 				if (x > it.x1 && x < it.x2 && y > it.y1 && y < it.y2) {
 					return it;
 				}
@@ -493,7 +505,7 @@ touchArea Keyboard::getKeyForCoordinates(int x, int y)
 	return { "", false, 0, 0, 0, 0 };
 }
 
-void Keyboard::setHighlightedKey(touchArea &area)
+void Keyboard::setHighlightedKey(const touchArea &area)
 {
 	highlightedKey = area;
 	isKeyHighlighted = true;
