@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2017-2021 Martijn Braam & Clayton Craft <clayton@craftyguy.net>
+Copyright (C) 2017-2022 Martijn Braam & Clayton Craft <clayton@craftyguy.net>
 
 This file is part of osk-sdl.
 
@@ -22,6 +22,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "config.h"
 #include <SDL2/SDL.h>
 #include <SDL2/SDL_ttf.h>
+#include <array>
 #include <cmath>
 #include <cstdint>
 #include <list>
@@ -53,8 +54,8 @@ struct rgb {
 };
 
 struct KeyboardLayer {
-	SDL_Texture *texture = nullptr;
-	SDL_Texture *highlightedTexture = nullptr;
+	SDL_Texture *texture;
+	SDL_Texture *highlightedTexture;
 	std::array<std::vector<std::string>, 4> rows;
 	std::vector<touchArea> keyVector;
 	int layerNum;
@@ -89,7 +90,7 @@ public:
 	  Set the key to be highlighted on the next render pass
 	  @param area Touch area of the key
 	  */
-	void setHighlightedKey(touchArea &area);
+	void setHighlightedKey(const touchArea &area);
 	/**
 	  Unset the key to be highlighted on the next render pass
 	  */
