@@ -31,6 +31,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <iostream>
 #include <list>
 #include <string>
+#include <sys/reboot.h>
 #include <unistd.h>
 
 bool lastUnlockingState = false;
@@ -90,7 +91,7 @@ int main(int argc, char **args)
 	/*
 	 * default to hiding the on-screen keyboard if a physical keyboard is present
 	 */
-	if (opts.noKeyboard || hasPhysKeyboard())
+	if (!opts.showKeyboard && (opts.noKeyboard || hasPhysKeyboard()))
 		show_osk = false;
 	SDL_LogInfo(SDL_LOG_CATEGORY_SYSTEM, "%sshowing on-screen keyboard", show_osk ? "" : "NOT ");
 
@@ -175,7 +176,7 @@ int main(int argc, char **args)
 	inputWidth = static_cast<int>(WIDTH * 0.9);
 	if (!show_osk) {
 		// Reduce height when no keyboard is shown
-		inputHeight = config.keyboardFontSize + 8;
+		inputHeight = config.inputBoxFontSize + 8;
 	} else {
 		inputHeight = static_cast<int>(WIDTH * 0.1);
 	}
@@ -331,6 +332,17 @@ int main(int argc, char **args)
 						continue;
 					}
 					break; // SDLK_BACKSPACE
+				case SDLK_POWER:
+					if (opts.testMode) {
+						SDL_LogInfo(SDL_LOG_CATEGORY_SYSTEM, "Power off requested, but ignoring because"
+							" test mode is active!");
+						break;
+					}
+					SDL_LogInfo(SDL_LOG_CATEGORY_SYSTEM, "Power off!");
+					sync();
+					reboot(RB_POWER_OFF);
+					SDL_LogError(SDL_LOG_CATEGORY_ERROR, "Failed to power off: %s", strerror(errno));
+					break;
 				case SDLK_ESCAPE:
 					goto QUIT;
 					break; // SDLK_ESCAPE
@@ -471,7 +483,8 @@ int main(int argc, char **args)
 				}
 				// If any animations are enabled and running, continue to push render events to the
 				// event queue
-				if (config.animations && (luksDev.unlockRunning() || keyboard.isInSlideAnimation())) {
+				bool keyboardIsAnimating = keyboard.isInSlideAnimation() && show_osk;
+				if (config.animations && (luksDev.unlockRunning() || keyboardIsAnimating)) {
 					SDL_PushEvent(&renderEvent);
 				}
 			}
