@@ -79,7 +79,7 @@ int Tooltip::init(SDL_Renderer *renderer, const std::string &text)
 		smooth_corners_surface(surface, SDL_MapRGBA(surface->format, 0, 0, 0, 0), &rect, cornerRadius);
 	}
 
-	TTF_Font *font = TTF_OpenFont(config->keyboardFont.c_str(), config->keyboardFontSize);
+	TTF_Font *font = TTF_OpenFont(config->keyboardFont.c_str(), config->inputBoxFontSize);
 	SDL_Surface *textSurface;
 	SDL_Color textColor = { foregroundColor.r, foregroundColor.g, foregroundColor.b, foregroundColor.a };
 	textSurface = TTF_RenderText_Blended(font, text.c_str(), textColor);
