@@ -34,10 +34,11 @@ int fetchOpts(int argc, char **args, Opts *opts)
 		{ "no-gles", no_argument, 0, 'G' },
 		{ "version", no_argument, 0, 'V' },
 		{ "no-keyboard", no_argument, 0, 'x' },
+		{ "show-keyboard", no_argument, 0, 's' },
 		{ 0, 0, 0, 0 }
 	};
 
-	while ((opt = getopt_long(argc, args, "td:n:c:o:kvGVx", longOpts, &optIndex)) != -1)
+	while ((opt = getopt_long(argc, args, "td:n:c:o:kvGVxs", longOpts, &optIndex)) != -1)
 		switch (opt) {
 		case 't':
 			opts->luksDevPath = DEFAULT_LUKSDEVPATH;
@@ -63,7 +64,10 @@ int fetchOpts(int argc, char **args, Opts *opts)
 			opts->verbose = true;
 			break;
 		case 'x':
-			opts->noKeyboard= true;
+			opts->noKeyboard = true;
+			break;
+		case 's':
+			opts->showKeyboard = true;
 			break;
 		case 'G':
 			opts->noGLES = true;
@@ -74,7 +78,8 @@ int fetchOpts(int argc, char **args, Opts *opts)
 		default:
 			SDL_LogError(SDL_LOG_CATEGORY_ERROR, "Usage: osk-sdl [-t|--testmode] [-k|--keyscript] [-d /dev/sda] [-n device_name] "
 												 "[-c /etc/osk.conf] [-o /boot/osk.conf] "
-												 "[-v|--verbose] [-G|--no-gles] [-x|--no-keyboard");
+												 "[-v|--verbose] [-G|--no-gles] [-x|--no-keyboard] "
+												 "[-s|--show-keyboard]");
 			return 1;
 		}
 	if (opts->luksDevPath.empty()) {
@@ -127,84 +132,40 @@ int find_gles_driver_index()
 	return -1;
 }
 
-SDL_Texture *circle = nullptr;
-int circleRadius = 0;
+SDL_Texture *dotGlyph = nullptr;
+int dotGlyphSize = 0;
 
-void draw_circle(SDL_Renderer *renderer, SDL_Point center, int radius, const argb &fillColor)
+void draw_dot_glyph(SDL_Renderer *renderer, SDL_Point center, int size, Config *config)
 {
+	if (config->inputBoxDotGlyph.compare("") == 0)
+		return;
+
 	// Destroy cached texture if radius doesn't match
-	if (circle && circleRadius != radius) {
-		SDL_LogWarn(SDL_LOG_CATEGORY_VIDEO, "Destroying previously cached circle texture with radius %i", circleRadius);
-		SDL_DestroyTexture(circle);
-		circle = nullptr;
-		circleRadius = 0;
+	if (dotGlyph && dotGlyphSize != size) {
+		SDL_LogWarn(SDL_LOG_CATEGORY_VIDEO, "Destroying previously cached dot glyph texture with radius %i",
+				dotGlyphSize);
+		SDL_DestroyTexture(dotGlyph);
+		dotGlyph = nullptr;
+		dotGlyphSize = 0;
 	}
 
 	// Cache a new texture if needed
-	if (!circle) {
-		SDL_LogInfo(SDL_LOG_CATEGORY_VIDEO, "Caching a new circle texture with radius %i", radius);
-		Uint32 rmask, gmask, bmask, amask;
+	if (!dotGlyph) {
+		SDL_LogInfo(SDL_LOG_CATEGORY_VIDEO, "Caching a new glyph texture with size %i", size);
+		TTF_Font *font = TTF_OpenFont(config->keyboardFont.c_str(), size);
+		SDL_Color textColor = { config->inputBoxForeground.r, config->inputBoxForeground.g, config->inputBoxForeground.b, config->inputBoxForeground.a };
+		SDL_Surface *textSurface = TTF_RenderUTF8_Blended(font, config->inputBoxDotGlyph.c_str(), textColor);
 
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-		rmask = 0xff000000;
-		gmask = 0x00ff0000;
-		bmask = 0x0000ff00;
-		amask = 0x000000ff;
-#else
-		rmask = 0x000000ff;
-		gmask = 0x0000ff00;
-		bmask = 0x00ff0000;
-		amask = 0xff000000;
-#endif
-
-		SDL_Surface *surface = SDL_CreateRGBSurface(0, 2 * radius, 2 * radius, 32, rmask, gmask, bmask, amask);
-		Uint32 color = SDL_MapRGB(surface->format, fillColor.r, fillColor.g, fillColor.b);
-		SDL_Rect rect;
-
-		int x0 = radius;
-		int y0 = radius;
-		int x = 0;
-		int y = radius;
-		float d = 5 / 4.0 - radius;
-
-		/*
-		* This is a version of the midpoint circle algorithm that draws rects
-		* between pairs of points for fill, rather than plotting every single
-		* pixel/point within the circle.
-		* https://stackoverflow.com/a/10878576
-		* https://stackoverflow.com/a/1201304
-		* */
-		while (x < y) {
-			if (d < 2 * (radius - y)) {
-				y -= 1;
-				d += 2 * y - 1;
-			} else if (d >= 2 * x) {
-				x += 1;
-				d -= 2 * x + 1;
-			} else {
-				x += 1;
-				y -= 1;
-				d += 2 * (y - x - 1);
-			}
-			rect = { x0 - y, y0 + x, 2 * y, 1 };
-			SDL_FillRect(surface, &rect, color);
-			rect = { x0 - x, y0 + y, 2 * x, 1 };
-			SDL_FillRect(surface, &rect, color);
-			rect = { x0 - x, y0 - y, 2 * x, 1 };
-			SDL_FillRect(surface, &rect, color);
-			rect = { x0 - y, y0 - x, 2 * y, 1 };
-			SDL_FillRect(surface, &rect, color);
-		}
-
-		circle = SDL_CreateTextureFromSurface(renderer, surface);
-		// note: don't disable alpha blending for this texture!
-		SDL_FreeSurface(surface);
-		circleRadius = radius;
+		dotGlyph = SDL_CreateTextureFromSurface(renderer, textSurface);
+		SDL_FreeSurface(textSurface);
+		dotGlyphSize = size;
 	}
 
 	// Copy cached texture into display
-	SDL_Rect rect = { center.x - radius, center.y - radius, 2 * radius, 2 * radius };
-	SDL_RenderCopy(renderer, circle, nullptr, &rect);
+	int w, h;
+	SDL_QueryTexture(dotGlyph, NULL, NULL, &w, &h);
+	SDL_Rect rect = { center.x - size / 2, center.y - size / 2, w, h};
+	SDL_RenderCopy(renderer, dotGlyph, nullptr, &rect);
 }
 
 void draw_password_box_dots(SDL_Renderer *renderer, Config *config, const SDL_Rect &inputRect, int numDots, bool busy)
@@ -213,7 +174,7 @@ void draw_password_box_dots(SDL_Renderer *renderer, Config *config, const SDL_Re
 	int ypos = inputRect.y + inputRect.h / 2;
 	int xmax = inputRect.x + inputRect.w;
 	float tick = static_cast<float>(SDL_GetTicks());
-	int dotSize = static_cast<int>(inputRect.h / 5);
+	int dotSize = static_cast<int>(inputRect.h / 2);
 	int padding = static_cast<int>(inputRect.h / 2);
 	int offset = 0;
 	/*
@@ -223,7 +184,7 @@ void draw_password_box_dots(SDL_Renderer *renderer, Config *config, const SDL_Re
 		SDL_RenderSetClipRect(renderer, &inputRect); // Prevent drawing outside the input bounds
 	for (int i = numDots - 1; i >= 0; i--) {
 		SDL_Point dotPos;
-		dotPos.x = inputRect.x + padding + (i * dotSize * 3) - offset;
+		dotPos.x = inputRect.x + padding + (i * dotSize) - offset;
 		// Offset dot center so that for long passwords the last dot aligns with right edge (minus padding)
 		if (dotPos.x + padding > xmax) {
 			offset = dotPos.x + padding - xmax;
@@ -238,7 +199,7 @@ void draw_password_box_dots(SDL_Renderer *renderer, Config *config, const SDL_Re
 		} else {
 			dotPos.y = ypos;
 		}
-		draw_circle(renderer, dotPos, dotSize, config->inputBoxForeground);
+		draw_dot_glyph(renderer, dotPos, dotSize, config);
 	}
 	if (!isDirectFB())
 		SDL_RenderSetClipRect(renderer, nullptr); // Reset clip rect
@@ -339,6 +300,11 @@ bool isDirectFB()
 
 bool hasPhysKeyboard()
 {
+	if (!std::filesystem::exists("/dev/input")) {
+		SDL_LogInfo(SDL_LOG_CATEGORY_SYSTEM, "/dev/input does not exist, skipping physical keyboard check");
+		return false;
+	}
+
 	for (const auto &file: std::filesystem::directory_iterator("/dev/input")) {
 
 		std::string dev_name = file.path().string();
@@ -361,8 +327,12 @@ bool hasPhysKeyboard()
 		ioctl(fd, EVIOCGBIT(EV_KEY, sizeof(keyMask)), &keyMask);
 		close(fd);
 
-		/* this contains ESC, numbers, and letters Q through D (and omits RESERVED keycode at bit 0)*/
-		unsigned long mask = 0xFFFFFFFE;
+		/*
+		 * This mask is the first 32-bits advertised by an N900 keyboard. It's obviously a physical keyboard, so
+		 * matching at *least* what it shows is a good baseline. Other physical keyboards should have no trouble
+		 * matching this.
+		 */
+		unsigned long mask = 0xF3FF4000;
 		if ((keyMask[0] & mask) == mask) {
 			SDL_LogInfo(SDL_LOG_CATEGORY_SYSTEM, "Probably a physical keyboard: %s", dev_name.c_str());
 			return true;
